@@ -84,8 +84,7 @@ void Category::parseData(const QString &path, QXmlStreamReader *xml)
             xml->readNext();
         } else if (xml->name() == QLatin1String("Icon")) {
             m_iconString = xml->readElementText();
-        } else if (xml->name() == QLatin1String("Include")
-                   || xml->name() == QLatin1String("Categories")) {
+        } else if (xml->name() == QLatin1String("Include") || xml->name() == QLatin1String("Categories")) {
             const QString opening = xml->name().toString();
             while (!xml->atEnd() && !xml->hasError()) {
                 xml->readNext();
@@ -210,7 +209,7 @@ void Category::setFilter(const CategoryFilter &filter)
     m_filter = filter;
 }
 
-QVector<Category *> Category::subCategories() const
+const QVector<Category *> &Category::subCategories() const
 {
     return m_subCategories;
 }
@@ -399,3 +398,5 @@ bool CategoryFilter::operator==(const CategoryFilter &other) const
         return std::get<QVector<CategoryFilter>>(value) == std::get<QVector<CategoryFilter>>(other.value);
     }
 }
+
+#include "moc_Category.cpp"
