@@ -4,27 +4,45 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef CATEGORY_H
-#define CATEGORY_H
+#pragma once
 
 #include <QObject>
 #include <QPair>
 #include <QSet>
+#include <QString>
 #include <QUrl>
 #include <QVector>
+#include <variant>
 
 #include "discovercommon_export.h"
 
-class QDomNode;
+class QXmlStreamReader;
 class QTimer;
 
-enum FilterType {
-    InvalidFilter,
-    CategoryFilter,
-    PkgSectionFilter,
-    PkgWildcardFilter,
-    PkgNameFilter,
-    AppstreamIdWildcardFilter,
+class CategoryFilter
+{
+    Q_GADGET
+public:
+    enum FilterType {
+        CategoryNameFilter,
+        PkgSectionFilter,
+        PkgWildcardFilter,
+        PkgNameFilter,
+        AppstreamIdWildcardFilter,
+        OrFilter,
+        AndFilter,
+        NotFilter,
+    };
+    Q_ENUM(FilterType)
+
+    FilterType type;
+    std::variant<QString, QVector<CategoryFilter>> value;
+
+    bool operator==(const CategoryFilter &other) const;
+    bool operator!=(const CategoryFilter &other) const
+    {
+        return !operator==(other);
+    }
 };
 
 class DISCOVERCOMMON_EXPORT Category : public QObject
@@ -39,7 +57,7 @@ public:
 
     Category(const QString &name,
              const QString &iconName,
-             const QVector<QPair<FilterType, QString>> &orFilters,
+             const CategoryFilter &filters,
              const QSet<QString> &pluginName,
              const QVector<Category *> &subCategories,
              bool isAddons);
@@ -50,11 +68,9 @@ public:
     // as the results could be potentially detremental to the function of the category filters
     void setName(const QString &name);
     QString icon() const;
-    QVector<QPair<FilterType, QString>> andFilters() const;
-    void setAndFilter(const QVector<QPair<FilterType, QString>> &filters);
-    QVector<QPair<FilterType, QString>> orFilters() const;
-    QVector<QPair<FilterType, QString>> notFilters() const;
-    QVector<Category *> subCategories() const;
+    void setFilter(const CategoryFilter &filter);
+    CategoryFilter filter() const;
+    const QVector<Category *> &subCategories() const;
     QVariantList subCategoriesVariant() const;
 
     static void sortCategories(QVector<Category *> &cats);
@@ -65,12 +81,16 @@ public:
      * root category to the global root category model.
      */
     void addSubcategory(Category *cat);
-    void parseData(const QString &path, const QDomNode &data);
+    void parseData(const QString &path, QXmlStreamReader *xml);
     bool blacklistPlugins(const QSet<QString> &pluginName);
     bool isAddons() const
     {
         return m_isAddons;
     }
+    qint8 priority() const
+    {
+        return m_priority;
+    }
     bool matchesCategoryName(const QString &name) const;
 
     Q_SCRIPTABLE bool contains(Category *cat) const;
@@ -79,22 +99,26 @@ public:
     static bool categoryLessThan(Category *c1, const Category *c2);
     static bool blacklistPluginsInVector(const QSet<QString> &pluginNames, QVector<Category *> &subCategories);
 
+    QStringList involvedCategories() const;
+    QString untranslatedName() const
+    {
+        return m_untranslatedName;
+    }
+
 Q_SIGNALS:
     void subCategoriesChanged();
     void nameChanged();
 
 private:
     QString m_name;
+    QString m_untranslatedName;
     QString m_iconString;
-    QVector<QPair<FilterType, QString>> m_andFilters;
-    QVector<QPair<FilterType, QString>> m_orFilters;
-    QVector<QPair<FilterType, QString>> m_notFilters;
+    CategoryFilter m_filter;
     QVector<Category *> m_subCategories;
 
-    QVector<QPair<FilterType, QString>> parseIncludes(const QDomNode &data);
+    CategoryFilter parseIncludes(QXmlStreamReader *xml);
     QSet<QString> m_plugins;
     bool m_isAddons = false;
+    qint8 m_priority = 0;
     QTimer *m_subCategoriesChanged;
 };
-
-#endif
