@@ -76,6 +76,7 @@ void FlatpakJobTransaction::start()
     connect(m_appJob, &FlatpakTransactionThread::passiveMessage, this, &FlatpakJobTransaction::passiveMessage);
     connect(m_appJob, &FlatpakTransactionThread::webflowStarted, this, &FlatpakJobTransaction::webflowStarted);
     connect(m_appJob, &FlatpakTransactionThread::webflowDone, this, &FlatpakJobTransaction::webflowDone);
+    connect(m_appJob, &FlatpakTransactionThread::proceedRequest, this, &FlatpakJobTransaction::proceedRequest);
 
     s_pool->start(m_appJob);
 }
@@ -88,8 +89,8 @@ void FlatpakJobTransaction::finishTransaction()
         m_app->setState(AbstractResource::None);
     }
 
-    if (!m_appJob->addedRepositories().isEmpty()) {
-        Q_EMIT repositoriesAdded(m_appJob->addedRepositories());
+    if (const auto repositories = m_appJob->addedRepositories(); !repositories.isEmpty()) {
+        Q_EMIT repositoriesAdded(repositories);
     }
 
     if (!m_appJob->cancelled() && !m_appJob->errorMessage().isEmpty()) {
@@ -102,3 +103,12 @@ void FlatpakJobTransaction::finishTransaction()
         setStatus(m_appJob->cancelled() ? CancelledStatus : DoneWithErrorStatus);
     }
 }
+
+void FlatpakJobTransaction::proceed()
+{
+    if (m_appJob) {
+        m_appJob->proceed();
+    }
+}
+
+#include "moc_FlatpakJobTransaction.cpp"
