@@ -5,12 +5,11 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef FLATPAKJOBTRANSACTION_H
-#define FLATPAKJOBTRANSACTION_H
+#pragma once
 
+#include "flatpak-helper.h"
 #include <QPointer>
 #include <Transaction/Transaction.h>
-#include "flatpak-helper.h"
 
 #include <gio/gio.h>
 #include <glib.h>
@@ -25,17 +24,22 @@ public:
 
     ~FlatpakJobTransaction();
 
+    Q_SCRIPTABLE void proceed() override;
     void cancel() override;
 
+    /** Mapping of repositories where a key is an installation path and a value is a list of names */
+    using Repositories = QMap<QString, QStringList>;
+
 public Q_SLOTS:
     void finishTransaction();
     void start();
 
+Q_SIGNALS:
+    void repositoriesAdded(const Repositories &repositories);
+
 private:
     void updateProgress();
 
     QPointer<FlatpakResource> m_app;
     QPointer<FlatpakTransactionThread> m_appJob;
 };
-
-#endif // FLATPAKJOBTRANSACTION_H
