@@ -7,11 +7,15 @@
 
 #include "FlatpakTransactionThread.h"
 #include "FlatpakResource.h"
+#include "libdiscover_backend_flatpak_debug.h"
 
 #include <KLocalizedString>
-#include <QDebug>
 #include <QDesktopServices>
 
+#include <span>
+
+using namespace Qt::StringLiterals;
+
 static int FLATPAK_CLI_UPDATE_FREQUENCY = 150;
 
 gboolean FlatpakTransactionThread::add_new_remote_cb(FlatpakTransaction *object,
@@ -21,7 +25,7 @@ gboolean FlatpakTransactionThread::add_new_remote_cb(FlatpakTransaction *object,
                                                      gchar *url,
                                                      gpointer user_data)
 {
-    FlatpakTransactionThread *obj = (FlatpakTransactionThread *)user_data;
+    auto obj = static_cast<FlatpakTransactionThread *>(user_data);
 
     // TODO ask instead
     auto name = QString::fromUtf8(suggested_remote_name);
@@ -32,7 +36,7 @@ gboolean FlatpakTransactionThread::add_new_remote_cb(FlatpakTransaction *object,
 
 void FlatpakTransactionThread::progress_changed_cb(FlatpakTransactionProgress *progress, gpointer user_data)
 {
-    FlatpakTransactionThread *obj = (FlatpakTransactionThread *)user_data;
+    auto obj = static_cast<FlatpakTransactionThread *>(user_data);
 
     g_autolist(GObject) ops = flatpak_transaction_get_operations(obj->m_transaction);
     g_autoptr(FlatpakTransactionOperation) op = flatpak_transaction_get_current_operation(obj->m_transaction);
@@ -54,7 +58,7 @@ void FlatpakTransactionThread::new_operation_cb(FlatpakTransaction * /*object*/,
                                                 FlatpakTransactionProgress *progress,
                                                 gpointer user_data)
 {
-    FlatpakTransactionThread *obj = (FlatpakTransactionThread *)user_data;
+    auto obj = static_cast<FlatpakTransactionThread *>(user_data);
 
     g_signal_connect(progress, "changed", G_CALLBACK(&FlatpakTransactionThread::progress_changed_cb), obj);
     flatpak_transaction_progress_set_update_frequency(progress, FLATPAK_CLI_UPDATE_FREQUENCY);
@@ -62,7 +66,7 @@ void FlatpakTransactionThread::new_operation_cb(FlatpakTransaction * /*object*/,
 
 void operation_error_cb(FlatpakTransaction * /*object*/, FlatpakTransactionOperation * /*operation*/, GError *error, gint /*details*/, gpointer user_data)
 {
-    FlatpakTransactionThread *obj = (FlatpakTransactionThread *)user_data;
+    auto obj = static_cast<FlatpakTransactionThread *>(user_data);
     if (error) {
         obj->addErrorMessage(QString::fromUtf8(error->message));
     }
@@ -75,8 +79,8 @@ FlatpakTransactionThread::webflowStart(FlatpakTransaction *transaction, const ch
     Q_UNUSED(options);
 
     QUrl webflowUrl(QString::fromUtf8(url));
-    qDebug() << "starting web flow" << webflowUrl << remote << id;
-    FlatpakTransactionThread *obj = (FlatpakTransactionThread *)user_data;
+    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "starting web flow" << webflowUrl << remote << id;
+    auto obj = static_cast<FlatpakTransactionThread *>(user_data);
     obj->m_webflows << id;
     Q_EMIT obj->webflowStarted(webflowUrl, id);
     return true;
@@ -86,12 +90,91 @@ void FlatpakTransactionThread::webflowDoneCallback(FlatpakTransaction *transacti
 {
     Q_UNUSED(transaction);
     Q_UNUSED(options);
-    FlatpakTransactionThread *obj = (FlatpakTransactionThread *)user_data;
+    auto obj = static_cast<FlatpakTransactionThread *>(user_data);
     obj->m_webflows << id;
     Q_EMIT obj->webflowDone(id);
-    qDebug() << "webflow done" << id;
+    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "webflow done" << id;
 }
 
+namespace Callbacks
+{
+void operation_done([[maybe_unused]] FlatpakTransaction *transaction,
+                    [[maybe_unused]] FlatpakTransactionOperation *operation,
+                    const char *commit,
+                    [[maybe_unused]] FlatpakTransactionResult details)
+{
+    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "NOOP" << commit;
+}
+
+int choose_remote_for_ref([[maybe_unused]] FlatpakTransaction *transaction, const char *for_ref, const char *runtime_ref, const char *const *remotes)
+{
+    const auto remotesCount = g_strv_length(const_cast<char **>(remotes));
+    if (LIBDISCOVER_BACKEND_FLATPAK_LOG().isDebugEnabled()) {
+        const auto remotesSpan = std::span{remotes, remotesCount};
+        QStringList remotesStrings;
+        remotesStrings.reserve(remotesCount);
+        for (const auto &remote : remotesSpan) {
+            remotesStrings.append(QString::fromUtf8(remote));
+        }
+        qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "NOOP" << for_ref << runtime_ref << remotesStrings;
+    }
+    // NOTE: this shouldn't actually happen for us, we always know the remote beforehand. If we fail the assertion here that indicates we have
+    // a problem earlier in the workflow producing multiple (unexpected) remotes.
+    Q_ASSERT(remotesCount <= 1);
+    return 0;
+}
+
+void end_of_lifed([[maybe_unused]] FlatpakTransaction *transaction, const char *ref, const char *reason, const char *rebase)
+{
+    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "NOOP" << ref << reason << rebase;
+}
+
+gboolean ready([[maybe_unused]] FlatpakTransaction *transaction)
+{
+    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "NOOP";
+    return true; // continue with transaction
+}
+
+gboolean run([[maybe_unused]] FlatpakTransaction *transaction, [[maybe_unused]] GCancellable *cancellable, [[maybe_unused]] GError **error)
+{
+    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "NOOP";
+    return true; // continue with transaction
+}
+
+gboolean end_of_lifed_with_rebase([[maybe_unused]] FlatpakTransaction *transaction,
+                                  const char *remote,
+                                  const char *ref,
+                                  const char *reason,
+                                  const char *rebased_to_ref,
+                                  const char **previous_ids,
+                                  gpointer user_data)
+{
+    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "end_of_lifed_with_rebase" << remote << ref << reason << rebased_to_ref << previous_ids;
+    return static_cast<FlatpakTransactionThread *>(user_data)->end_of_lifed_with_rebase(remote, ref, reason, rebased_to_ref, previous_ids);
+}
+
+gboolean basic_auth_start([[maybe_unused]] FlatpakTransaction *transaction,
+                          const char *remote,
+                          const char *realm,
+                          [[maybe_unused]] GVariant *options,
+                          [[maybe_unused]] guint id)
+{
+    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "NOOP" << remote << realm;
+    return true; // continue with transaction
+}
+
+void install_authenticator([[maybe_unused]] FlatpakTransaction *transaction, const char *remote, const char *authenticator_ref)
+{
+    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "NOOP" << remote << authenticator_ref;
+}
+
+gboolean ready_pre_auth([[maybe_unused]] FlatpakTransaction *transaction)
+{
+    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "NOOP";
+    return true; // continue with transaction
+}
+} // namespace Callbacks
+
 FlatpakTransactionThread::FlatpakTransactionThread(FlatpakResource *app, Transaction::Role role)
     : m_result(false)
     , m_app(app)
@@ -103,12 +186,21 @@ FlatpakTransactionThread::FlatpakTransactionThread(FlatpakResource *app, Transac
     m_transaction = flatpak_transaction_new_for_installation(app->installation(), m_cancellable, &localError);
     if (localError) {
         addErrorMessage(QString::fromUtf8(localError->message));
-        qWarning() << "Failed to create transaction" << m_errorMessage;
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to create transaction" << m_errorMessage;
     } else {
         g_signal_connect(m_transaction, "add-new-remote", G_CALLBACK(add_new_remote_cb), this);
         g_signal_connect(m_transaction, "new-operation", G_CALLBACK(new_operation_cb), this);
         g_signal_connect(m_transaction, "operation-error", G_CALLBACK(operation_error_cb), this);
 
+        g_signal_connect(m_transaction, "basic-auth-start", G_CALLBACK(Callbacks::basic_auth_start), this);
+        g_signal_connect(m_transaction, "choose-remote-for-ref", G_CALLBACK(Callbacks::choose_remote_for_ref), this);
+        g_signal_connect(m_transaction, "end-of-lifed", G_CALLBACK(Callbacks::end_of_lifed), this);
+        g_signal_connect(m_transaction, "end-of-lifed-with-rebase", G_CALLBACK(Callbacks::end_of_lifed_with_rebase), this);
+        g_signal_connect(m_transaction, "install-authenticator", G_CALLBACK(Callbacks::install_authenticator), this);
+        g_signal_connect(m_transaction, "operation-done", G_CALLBACK(Callbacks::operation_done), this);
+        g_signal_connect(m_transaction, "ready", G_CALLBACK(Callbacks::ready), this);
+        g_signal_connect(m_transaction, "ready-pre-auth", G_CALLBACK(Callbacks::ready_pre_auth), this);
+
         if (qEnvironmentVariableIntValue("DISCOVER_FLATPAK_WEBFLOW")) {
             g_signal_connect(m_transaction, "webflow-start", G_CALLBACK(webflowStart), this);
             g_signal_connect(m_transaction, "webflow-done", G_CALLBACK(webflowDoneCallback), this);
@@ -124,8 +216,13 @@ FlatpakTransactionThread::~FlatpakTransactionThread()
 
 void FlatpakTransactionThread::cancel()
 {
-    for (int id : std::as_const(m_webflows))
+    QMutexLocker lock(&m_proceedMutex);
+    m_proceed = false;
+    m_proceedCondition.wakeAll();
+
+    for (int id : std::as_const(m_webflows)) {
         flatpak_transaction_abort_webflow(m_transaction, id);
+    }
     g_cancellable_cancel(m_cancellable);
 }
 
@@ -135,8 +232,9 @@ void FlatpakTransactionThread::run()
         Q_EMIT finished();
     });
 
-    if (!m_transaction)
+    if (!m_transaction) {
         return;
+    }
     g_autoptr(GError) localError = nullptr;
 
     const QString refName = m_app->ref();
@@ -145,10 +243,13 @@ void FlatpakTransactionThread::run()
         bool correct = false;
         if (m_app->state() == AbstractResource::Upgradeable && m_app->isInstalled()) {
             correct = flatpak_transaction_add_update(m_transaction, refName.toUtf8().constData(), nullptr, nullptr, &localError);
+            for (const QByteArray &subref : m_app->toUpdate()) {
+                correct = correct && flatpak_transaction_add_update(m_transaction, subref.constData(), nullptr, nullptr, &localError);
+            }
         } else if (m_app->flatpakFileType() == FlatpakResource::FileFlatpak) {
             g_autoptr(GFile) file = g_file_new_for_path(m_app->resourceFile().toLocalFile().toUtf8().constData());
             if (!file) {
-                qWarning() << "Failed to install bundled application" << refName;
+                qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to install bundled application" << refName;
                 m_result = false;
                 return;
             }
@@ -156,7 +257,7 @@ void FlatpakTransactionThread::run()
         } else if (m_app->flatpakFileType() == FlatpakResource::FileFlatpakRef && m_app->resourceFile().isLocalFile()) {
             g_autoptr(GFile) file = g_file_new_for_path(m_app->resourceFile().toLocalFile().toUtf8().constData());
             if (!file) {
-                qWarning() << "Failed to install flatpakref application" << refName;
+                qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to install flatpakref application" << refName;
                 m_result = false;
                 return;
             }
@@ -171,11 +272,7 @@ void FlatpakTransactionThread::run()
         }
 
         if (!correct) {
-            m_result = false;
-            m_errorMessage = QString::fromUtf8(localError->message);
-            // We are done so we can set the progress to 100
-            setProgress(100);
-            qWarning() << "Failed to install" << m_app->flatpakFileType() << refName << ':' << m_errorMessage;
+            fail(qUtf8Printable(refName), localError);
             return;
         }
     } else if (m_role == Transaction::Role::RemoveRole) {
@@ -184,7 +281,7 @@ void FlatpakTransactionThread::run()
             m_errorMessage = QString::fromUtf8(localError->message);
             // We are done so we can set the progress to 100
             setProgress(100);
-            qWarning() << "Failed to uninstall" << refName << ':' << m_errorMessage;
+            qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to uninstall" << refName << ':' << m_errorMessage;
             return;
         }
     }
@@ -202,22 +299,22 @@ void FlatpakTransactionThread::run()
         g_autoptr(GError) refsError = nullptr;
         g_autoptr(GPtrArray) refs = flatpak_installation_list_unused_refs(installation, nullptr, m_cancellable, &refsError);
         if (!refs) {
-            qWarning() << "could not fetch unused refs" << refsError->message;
+            qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "could not fetch unused refs" << refsError->message;
         } else if (refs->len > 0) {
             g_autoptr(GError) localError = nullptr;
-            qDebug() << "found unused refs:" << refs->len;
+            qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "found unused refs:" << refs->len;
             auto transaction = flatpak_transaction_new_for_installation(installation, m_cancellable, &localError);
             for (uint i = 0; i < refs->len; i++) {
                 FlatpakRef *ref = FLATPAK_REF(g_ptr_array_index(refs, i));
                 g_autofree gchar *strRef = flatpak_ref_format_ref(ref);
-                qDebug() << "unused ref:" << strRef;
+                qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "unused ref:" << strRef;
                 if (!flatpak_transaction_add_uninstall(transaction, strRef, &localError)) {
-                    qDebug() << "failed to uninstall unused ref" << refName << localError->message;
+                    qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "failed to uninstall unused ref" << refName << localError->message;
                     break;
                 }
             }
             if (!flatpak_transaction_run(transaction, m_cancellable, &localError)) {
-                qWarning() << "could not properly clean the elements" << refs->len << localError->message;
+                qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "could not properly clean the elements" << refs->len << localError->message;
             }
         }
 #endif
@@ -226,11 +323,16 @@ void FlatpakTransactionThread::run()
     setProgress(100);
 }
 
+int FlatpakTransactionThread::progress() const
+{
+    return m_progress;
+}
+
 void FlatpakTransactionThread::setProgress(int progress)
 {
     Q_ASSERT(qBound(0, progress, 100) == progress);
     if (m_progress > progress) {
-        qDebug() << "progress is regressing :(" << m_progress << progress;
+        qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "progress is regressing :(" << m_progress << progress;
         return;
     }
 
@@ -260,8 +362,9 @@ bool FlatpakTransactionThread::result() const
 
 void FlatpakTransactionThread::addErrorMessage(const QString &error)
 {
-    if (!m_errorMessage.isEmpty())
+    if (!m_errorMessage.isEmpty()) {
         m_errorMessage.append(QLatin1Char('\n'));
+    }
     m_errorMessage.append(error);
 }
 
@@ -269,3 +372,62 @@ bool FlatpakTransactionThread::cancelled() const
 {
     return g_cancellable_is_cancelled(m_cancellable);
 }
+
+FlatpakTransactionThread::Repositories FlatpakTransactionThread::addedRepositories() const
+{
+    return m_addedRepositories;
+}
+
+void FlatpakTransactionThread::fail(const char *refName, GError *error)
+{
+    m_result = false;
+    m_errorMessage = QString::fromUtf8(error->message);
+    // We are done so we can set the progress to 100
+    setProgress(100);
+    qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to install" << m_app->flatpakFileType() << refName << ':' << m_errorMessage;
+}
+
+bool FlatpakTransactionThread::end_of_lifed_with_rebase(const char *remote,
+                                                        const char *ref,
+                                                        const char *reason,
+                                                        const char *rebased_to_ref,
+                                                        const char **previous_ids)
+{
+    QMutexLocker lock(&m_proceedMutex);
+
+    if (QString::fromUtf8(rebased_to_ref).startsWith("runtime/"_L1)) {
+        qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Automatically transitioning runtime";
+        m_proceed = true;
+    } else {
+        m_proceed = false;
+        Q_EMIT proceedRequest(
+            i18nc("@title", "Replacement Available"),
+            xi18nc(
+                "@info %1 and 2 are flatpak ids e.g. org.kde.krita (can be rather lengthy though)",
+                "<resource>%1</resource> is no longer receiving updates.<nl/><nl/>Replace it with the supported version provided by <resource>%2</resource>?",
+                QString::fromUtf8(ref),
+                QString::fromUtf8(rebased_to_ref)));
+        m_proceedCondition.wait(&m_proceedMutex);
+    }
+
+    if (!m_proceed) {
+        return false;
+    }
+
+    GError *localError = nullptr;
+    const auto correct = flatpak_transaction_add_rebase_and_uninstall(m_transaction, remote, rebased_to_ref, ref, nullptr, previous_ids, &localError);
+    if (!correct || localError) {
+        fail(ref, localError);
+        return false;
+    }
+    return m_proceed;
+}
+
+void FlatpakTransactionThread::proceed()
+{
+    QMutexLocker lock(&m_proceedMutex);
+    m_proceed = true;
+    m_proceedCondition.wakeAll();
+}
+
+#include "moc_FlatpakTransactionThread.cpp"
