@@ -12,9 +12,11 @@
 #include <glib.h>
 
 #include <QMap>
+#include <QMutex>
 #include <QRunnable>
 #include <QStringList>
 #include <QThread>
+#include <QWaitCondition>
 #include <Transaction/Transaction.h>
 
 class FlatpakResource;
@@ -25,13 +27,11 @@ public:
     FlatpakTransactionThread(FlatpakResource *app, Transaction::Role role);
     ~FlatpakTransactionThread() override;
 
+    void proceed();
     void cancel();
     void run() override;
 
-    int progress() const
-    {
-        return m_progress;
-    }
+    int progress() const;
     void setProgress(int progress);
     void setSpeed(quint64 speed);
 
@@ -40,10 +40,11 @@ public:
     bool cancelled() const;
 
     void addErrorMessage(const QString &error);
-    QMap<QString, QStringList> addedRepositories() const
-    {
-        return m_addedRepositories;
-    }
+
+    /** Mapping of repositories where a key is an installation path and a value is a list of names */
+    using Repositories = QMap<QString, QStringList>;
+    Repositories addedRepositories() const;
+    [[nodiscard]] bool end_of_lifed_with_rebase(const char *remote, const char *ref, const char *reason, const char *rebased_to_ref, const char **previous_ids);
 
 Q_SIGNALS:
     void progressChanged(int progress);
@@ -52,6 +53,7 @@ Q_SIGNALS:
     void webflowStarted(const QUrl &url, int id);
     void webflowDone(int id);
     void finished();
+    void proceedRequest(const QString &title, const QString &description);
 
 private:
     static gboolean
@@ -59,6 +61,7 @@ private:
     static void progress_changed_cb(FlatpakTransactionProgress *progress, gpointer user_data);
     static void
     new_operation_cb(FlatpakTransaction * /*object*/, FlatpakTransactionOperation *operation, FlatpakTransactionProgress *progress, gpointer user_data);
+    void fail(const char *refName, GError *error);
 
     static gboolean webflowStart(FlatpakTransaction *transaction, const char *remote, const char *url, GVariant *options, guint id, gpointer user_data);
     static void webflowDoneCallback(FlatpakTransaction *transaction, GVariant *options, guint id, gpointer user_data);
@@ -72,6 +75,9 @@ private:
     FlatpakResource *const m_app;
     const Transaction::Role m_role;
     QMap<QString, QStringList> m_addedRepositories;
+    QMutex m_proceedMutex;
+    QWaitCondition m_proceedCondition;
+    bool m_proceed = false;
 
     QVector<int> m_webflows;
 };
