@@ -94,10 +94,7 @@ public:
     QVector<AbstractResource *> extendedBy(const QString &id) const;
 
     PKResolveTransaction *resolvePackages(const QStringList &packageNames);
-    void fetchDetails(const QString &pkgid)
-    {
-        m_details.add(pkgid);
-    }
+    void fetchDetails(const QString &pkgid);
     void fetchDetails(const QSet<QString> &pkgid);
 
     void checkForUpdates() override;
@@ -161,7 +158,7 @@ private:
 
     QVector<StreamResult> resultsByComponents(const AppStream::ComponentBox &names) const;
 
-    void runWhenInitialized(const std::function<void()> &f, QObject *stream);
+    PKResultsStream *deferredResultStream(const QString &streamName, std::function<void(PKResultsStream *)> callback);
 
     void checkDaemonRunning();
     void acquireFetching(bool f);
