@@ -24,6 +24,8 @@ SnapTransaction::SnapTransaction(QSnapdClient *client, SnapResource *app, Role r
 {
     if (role == RemoveRole)
         setRequest(m_client->remove(app->packageName()));
+    else if (app->state() == AbstractResource::Upgradeable)
+        setRequest(m_client->refresh(app->packageName()));
     else
         setRequest(m_client->install(app->packageName()));
 }
@@ -123,3 +125,5 @@ void SnapTransaction::progressed()
     setProgress(percentage / qMax(count, 1));
     setStatus(status);
 }
+
+#include "moc_SnapTransaction.cpp"
